package org.greenstone.gatherer.gui;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import calpa.html.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.util.GURL;
import org.greenstone.gatherer.util.Utility;

public class PreviewPane
    extends JPanel {

    private CalHTMLPane view   = null;
    private int back_count     = 0;
    private int forward_count  = 0;
    private JButton back       = null;
    private JButton forward    = null;
    private JButton home       = null;
    private JButton reload = null;
    private JLabel status      = null;
    private Observer observer = null;
    private URL homepage = null;

    static final public String BLANK_PAGE = "<html><head><title>No Page</title></head><body></body></html>";

    public PreviewPane() {
	super();

	CalHTMLManager.setCacheDocuments(false);

	// Create components
	back = new JButton(get("Back"), Utility.getImage("back.gif"));
	back.addActionListener(new BackListener());
	back.setEnabled(false);

	home = new JButton(get("Home"), Utility.getImage("home.gif"));
	home.addActionListener(new HomeListener());
	home.setEnabled(false);

	forward = new JButton(get("Forward"), Utility.getImage("forward.gif"));
	forward.addActionListener(new ForwardListener());
	forward.setEnabled(false);

	reload = new JButton(get("Reload"), Utility.getImage("reload.gif"));
	reload.addActionListener(new ReloadListener());
	reload.setEnabled(false);

	CalHTMLPreferences prefs = new CalHTMLPreferences();
	observer = new Observer();
	view = new CalHTMLPane(prefs, observer, "Default");

	status = new JLabel(get("Ready"));
    }

    public void collectionChanged(boolean ready) {
	// If we were showing a page, but that page is no longer available, then return to a blank page
	if(!ready && homepage != null) {
	    homepage = null;
	    view.showHTMLDocument(BLANK_PAGE);
	}
	if (ready) {
	    // reload the home page for the current collection
	    try {
		// Now load the collection
		homepage = new URL(Gatherer.config.exec_address.toString() + "?a=p&p=about&c=" + Gatherer.c_man.getCollection().getName());
		///atherer.println("Loading " + url);
		status.setText(get("Loading", homepage.toString()));
		view.showHTMLDocument(homepage, null, true);
	    }
	    catch (MalformedURLException exception) {
		Gatherer.printStackTrace(exception);
	    }
	}
    }

    public void display() {
	JPanel control_pane = new JPanel();
	control_pane.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	control_pane.setLayout(new GridLayout(1,4));
	control_pane.add(back);
	control_pane.add(reload);
	control_pane.add(home);
	control_pane.add(forward);

	JPanel a_panel = new JPanel(new BorderLayout());
	a_panel.setSize(new Dimension(770, 440));
	a_panel.add(view, BorderLayout.CENTER);

	setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	setLayout(new BorderLayout());
	add(control_pane, BorderLayout.NORTH);
	add(a_panel, BorderLayout.CENTER);
	add(status, BorderLayout.SOUTH);
    }

    /** This method is called when the user selects the 'Preview' tab from the views bar. Can be used to display a particular page or perform some other relevant tests (ensure internet connection to specified server maybe?)
     */
    public void gainFocus() {
	if(Gatherer.c_man.ready() && Gatherer.c_man.built()) {
	    if(homepage == null) {
		try {
		    // Now load the collection
		    homepage = new URL(Gatherer.config.exec_address.toString() + "?a=p&p=about&c=" + Gatherer.c_man.getCollection().getName());
		    ///atherer.println("Loading " + url);
		    status.setText(get("Loading", homepage.toString()));
		    view.showHTMLDocument(homepage);
		}
		catch (MalformedURLException exception) {
		    Gatherer.printStackTrace(exception);
		}
	    }
	}
	validate();
    }

    /** Used to retrieve the phrase that matches the given key from the loaded dictionary.
     */
    public String get(String key) {
	return get(key, null);
    }

    public String get(String key, String arg) {
	if(key.indexOf(".") == -1) {
	    key = "Browser." + key;
	}
	return Gatherer.dictionary.get(key, arg);
    }

    public void configServer(String command) {
	try {
	    String url = Gatherer.config.exec_address.toString() + command;
	    ///ystem.err.println("Action: " + url);
	    view.setLoadSynchronously(true);
	    view.showHTMLDocument(new URL(url));
	    view.setLoadSynchronously(false);
	    ///ystem.err.println("Complete.");
	    url = null;		  
	}
	catch(MalformedURLException error) {
	    ///ystem.err.println("Bad URL.");
	}
    }

    public void validate() {
	back.setEnabled(back_count > 0);
	home.setEnabled(homepage != null);
	forward.setEnabled(forward_count > 0);
	reload.setEnabled(homepage != null);
    }

    private class BackListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view.goBack();
	    back_count--;
	    forward_count++;
	    validate();
	}
    }

    private class ForwardListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view.goForward();
	    back_count++;
	    forward_count--;
	    validate();
	}
    }

    private class HomeListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view.showHTMLDocument(homepage);
	    back_count++;
	    forward_count = 0;
	    validate();
	}
    }

    private class ReloadListener
	implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    view.reloadDocument();
	}
    }

    private class Observer
	extends DefaultCalHTMLObserver {
	public int state = CalCons.DOC_LOADED;
	public void linkActivatedUpdate(CalHTMLPane pane, URL url, String target_frame, String j_name) {
	    back_count++;
	    forward_count = 0;
	    validate();
	}
	public void linkFocusUpdate(CalHTMLPane pane, URL url) {
	    ///ystem.err.println("Link in focus " + url.toString());
	    status.setText(get("Follow", url.toString()));
	}
	public void statusUpdate(CalHTMLPane pane, int state, URL url, int value, String message) {
	    this.state = state;
	    switch(state) {
		// The Pane is attempting to connect to the given URL to receive data.
	    case CalCons.PRE_CONNECT:
		///ystem.err.println("Preconnect: " + state + " - " + message);
		break;
		// The Pane was unable to connect to the given URL or was unable to parse the content. Most likely this will be due to an incorrectly specified URL. The message argument may contain further details of the reason for failure.
	    case CalCons.PARSE_FAILED:
		///ystem.err.println("Parse Failed: " + state + " - " + message);
		status.setText(get("CannotConnect", url.toString()));
		break;
		// The Pane has established a connection to the given URL and is receiving any content.
	    case CalCons.CONNECTED:
		///ystem.err.println("Connected: " + state + " - " + message);
		status.setText(get("Loading", url.toString()));
		break;
		// The size of the content at the given URL is known and is contained in the value argument.
	    case CalCons.DOC_LENGTH:
		///ystem.err.println("Doc Length: " + state + " - " + message);
		break;
		// The title of the document for the given URL is known and is contained in the message argument. Only the title of a document in the Pane's top level frame will be sent to this method. If the document has no name, the message will be null, unless the document is a Frameset document in which case the message "Frameset" will be sent.
	    case CalCons.TITLE:
		///ystem.err.println("Title: " + state + " - " + message);
		break;
		// An exception has been thrown during parsing of the data from the URL. This will most likely be an IOException such as a server time-out.
	    case CalCons.PARSE_FAILED_POST_CONNECT:
		///ystem.err.println("Parse Failed Post Connect: " + state + " - " + message);
		status.setText(get("TimedOut", url.toString()));
		break;
		// The document has been parsed but formatting cannot be completed because the document contains images of unspecified size. The parsing thread is waiting for image updates to give it the information it needs to format and display the document.
	    case CalCons.WAITING_FOR_IMAGES:
		///ystem.err.println("Waiting For Images: " + state + " - " + message + " - " + url.toString());
		break;
		// All text and image data has been received, parsed and the document structure determined.
	    case CalCons.DOC_LOADED:
		///ystem.err.println("Doc Loaded: " + state + " - " + message);
		status.setText(get("Ready"));
		break;
	    }
	}
    }
}
