/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




/* GPL_HEADER */
package org.greenstone.gatherer.gui;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Company:      The University of Waikato
 * Written:      19/09/02
 * Revised:      
 * @author John Thompson, 9826509
 * @version 2.3
 **************************************************************************************/

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.lang.Runnable;
import java.lang.Thread;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import org.greenstone.gatherer.Gatherer;

public class SaveProgressDialog
    extends Thread {
	 
    private Gatherer gatherer;
    private JDialog dialog;
    private JProgressBar progress;
    static final public int METADATA = 50;
    static final public int COLLECTION = 40;
    static final public int MISC = 10;
    static final private Dimension SIZE = new Dimension(300,30);

    public SaveProgressDialog(Gatherer gatherer) {
	this.gatherer = gatherer;
	// Creation
	dialog = new JDialog();
	dialog.setUndecorated(true);
	dialog.setModal(true);
	dialog.setSize(SIZE);
	JPanel content_pane = (JPanel)dialog.getContentPane();
	progress = new JProgressBar();
	progress.setMinimum(0);
	progress.setMaximum(100);
	progress.setString(gatherer.get("SaveProgressDialog.Saving", "0"));
	progress.setStringPainted(true);
	// Layout
	content_pane.setLayout(new BorderLayout());
	content_pane.add(progress, BorderLayout.CENTER);
	Dimension screen_size = gatherer.config.screen_size;
	dialog.setLocation((screen_size.width - SIZE.width) / 2, (screen_size.height - SIZE.height) / 2);
    }

    public void dispose() {
	dialog.dispose();
    }

    public void run() {
	dialog.show(); // Modal.
    }

    public void update(final int amount) {
	final JProgressBar final_progress = progress;
	final Runnable update = new Runnable() {
		public void run() {
		    progress.setValue(final_progress.getValue() + amount);
		}
	    };
	try {
	    SwingUtilities.invokeLater(update);
	}
	catch (Exception e) {
	    Gatherer.println("Error in SaveProgressDialog.update(): " + e);
	    Gatherer.printStackTrace(e);
	}
    }
}
