package org.greenstone.gatherer.valuetree;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.gui.MetaEditPane;
import org.greenstone.gatherer.msm.ElementWrapper;
import org.greenstone.gatherer.util.Utility;
/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.2
 */
public class GValueTree
    extends JPanel
    implements TreeSelectionListener {
    private boolean ignore_tree_selection_event = false;
    private boolean manual_text_edit_event;
    private CardLayout card_layout;
    private String card_showing = null;
    /** The metadata element that is currently selected. */
    private ElementWrapper selected_metadata_element = null;
    private GValueModel vm;
    private JTextArea extracted_message;
    private JTextField value;
    private JTree tree;
    private MetaEditPane metaedit;
    private String args[] = new String[1];
    /** Stock standard size for labels. */
    static final private Dimension VALUE_LABEL_SIZE = new Dimension(66, 26);

    static final private String NONE = "None";
    static final private String TREE = "Tree";


    public GValueTree(MetaEditPane metaedit, int width, int height, JButton add, JButton update, JButton remove, JButton expand) {
	super();

	this.metaedit = metaedit;

	setFont(Gatherer.config.getFont("general.font", false));
	setPreferredSize(new Dimension(width, height));

	JPanel metadata_pane = new JPanel();

	JPanel value_pane = new JPanel();
	JLabel value_label = new JLabel(get("Value"));
	value_label.setPreferredSize(VALUE_LABEL_SIZE);

	JPanel value_field_pane = new JPanel();
	value = new JTextField();
	value.setBackground(Gatherer.config.getColor("coloring.editable", false));
	value.setPreferredSize(new Dimension(413, 24));
	value.getDocument().addDocumentListener(new DocumentListenerImpl());

	JPanel button_pane = new JPanel();

	JPanel tree_pane = new JPanel();
	JLabel tree_label = new JLabel(get("Tree"));

	tree = new JTree(new GValueModel());
	tree.addTreeSelectionListener(this);
	tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	tree.setRootVisible(false);
	tree.putClientProperty("JTree.lineStyle", "Angled");

	JPanel extracted_pane = new JPanel();
	extracted_message = new JTextArea("");
	extracted_message.setEditable(false);
	extracted_message.setLineWrap(true);
	extracted_message.setOpaque(false);
	extracted_message.setWrapStyleWord(true);

	card_layout = new CardLayout();

	// Layout

	value_field_pane.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
	value_field_pane.setLayout(new BorderLayout(0, 0));
	value_field_pane.add(value, BorderLayout.CENTER);

	button_pane.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	button_pane.setLayout(new GridLayout());
	button_pane.add(add);
	button_pane.add(update);
	button_pane.add(remove);

	value_pane.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	value_pane.setLayout(new BorderLayout());
	value_pane.add(value_label, BorderLayout.WEST);
	value_pane.add(value_field_pane, BorderLayout.CENTER);
	value_pane.add(expand, BorderLayout.EAST);
	value_pane.add(button_pane, BorderLayout.SOUTH);

	tree_pane.setLayout(new BorderLayout());
	tree_pane.add(tree_label, BorderLayout.NORTH);
	tree_pane.add(new JScrollPane(tree), BorderLayout.CENTER);

	metadata_pane.setLayout(new BorderLayout());
	metadata_pane.add(value_pane, BorderLayout.NORTH);
	metadata_pane.add(tree_pane, BorderLayout.CENTER);

	extracted_pane.setBorder(BorderFactory.createEmptyBorder(25,10,25,10));
	extracted_pane.setLayout(new BorderLayout());
	extracted_pane.add(extracted_message, BorderLayout.CENTER);

	this.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	this.setLayout(card_layout);
	this.add(metadata_pane, TREE);
	this.add(extracted_pane, NONE);
	card_showing = TREE;
    }


    public GValueModel getModel() {
	return (GValueModel) tree.getModel();
    }


    public ElementWrapper getSelectedMetadataElement() {
	return selected_metadata_element;
    }


    public String getSelectedValue() {
	return value.getText();
    }


    public void setSelectedMetadataElement(ElementWrapper element)
    {
	// System.err.println("Setting selected metadata element to: " + element);
	if (element == null) {
	    selected_metadata_element = null;
	    return;
	}

	// Don't bother if the element selected is the same as the current one
	if (selected_metadata_element != null && element.equals(selected_metadata_element)) {
	    return;
	}

	if (vm != null) {
	    // Close any expanded paths in the tree
	    GValueNode root_node = (GValueNode) vm.getRoot();
	    for (int i = 0; i < root_node.getChildCount(); i++) {
		GValueNode child_node = (GValueNode) root_node.getChildAt(i);
		TreePath child_path = new TreePath(child_node.getPath());
		if (tree.isExpanded(child_path)) {
		    tree.collapsePath(child_path);
		}
	    }
	}

	// Load the model for the newly selected metadata element
	selected_metadata_element = element;
	vm = Gatherer.c_man.getCollection().msm.getValueTree(element);
	tree.setModel(vm);

	// Special case the extracted metadata set
	if (selected_metadata_element.getNamespace().equals("")) {
	    // Display the panel showing the "you cannot edit this metadata" message
	    args[0] = selected_metadata_element.toString();
	    extracted_message.setText(get("AutoMessage", args));
	    card_layout.show(this, NONE);
	    card_showing = NONE;
	}
	else {
	    // Display the panel for viewing and assigning metadata
	    card_layout.show(this, TREE);
	    card_showing = TREE;
	}
    }


    /** Handles TreeSelectionEvents. */
    public void valueChanged(TreeSelectionEvent event)
    {
	if(tree.getSelectionCount() != 0 && !ignore_tree_selection_event) {
	    TreePath path = tree.getSelectionPath();
	    GValueNode node = (GValueNode) path.getLastPathComponent();
	    setSelectedValue(node.getFullPath());
	}
    }


    /** Sets the value in the text field. */
    public void setSelectedValue(String val)
    {
	// Setting the text of the field causes the DocumentListener to be notified, and
	//   updating the tree is handled there (DocumentListenerImpl::validate())
	// System.err.println("Setting selected value to: " + val);
	if (!card_showing.equals(NONE)) {
	    manual_text_edit_event = val.equals("");  // Set to false unless val == ""
	    value.setText(Utility.stripNL(val));
	    value.setCaretPosition(0);
	    manual_text_edit_event = true;
	}
    }


    private String get(String key) {
	return get(key, null);
    }

    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "MetaEdit." + key;
	}
	return Gatherer.dictionary.get(key,args);
    }


    /** Returns a TreePath for the node most closely matching the metadata value. */
    private TreePath getClosestPath(String val)
    {
	// Start at the root of the tree
	GValueNode tree_node = (GValueNode) tree.getModel().getRoot();

	// Hierarchical values are separated using '\'
	StringTokenizer tokenizer = new StringTokenizer(val, "\\");
	while (tokenizer.hasMoreTokens()) {
	    String token = tokenizer.nextToken();

	    // All components except the last must match exactly
	    if (tokenizer.hasMoreTokens()) {
		for (int i = 0; i < tree_node.getChildCount(); i++) {
		    GValueNode child_node = (GValueNode) tree_node.getChildAt(i);
		    if (child_node.toString().equals(token)) {
			// The matching node has been found, so move onto the next token
			tree_node = child_node;
			break;
		    }
		}
	    }

	    // The last component may match partially
	    else {
		for (int i = 0; i < tree_node.getChildCount(); i++) {
		    GValueNode child_node = (GValueNode) tree_node.getChildAt(i);
		    if (child_node.toString().startsWith(token)) {
			// The closest node has been found, so return its path
			return new TreePath(child_node.getPath());
		    }
		}

		// Not even a partial match
		return null;
	    }
	}

	// If nothing else, return the path of the root node
	return new TreePath(tree_node.getPath());
    }


    /** This function is copied from JTree::setPathToVisible(), and modified so tree rows
	are always shown flushed left. Note: doesn't do accessibleContext stuff. */
    private void scrollTreePathToVisible(TreePath path)
    {
	if (path != null) {
	    tree.makeVisible(path);

	    Rectangle bounds = tree.getPathBounds(path);
	    if (bounds != null) {
		bounds.width += bounds.x;
		bounds.x = 0;
		tree.scrollRectToVisible(bounds);
	    }
	}
    }


    private class DocumentListenerImpl
	implements DocumentListener {
	/** Gives notification that an attribute or set of attributes changed. */
	public void changedUpdate(DocumentEvent e) {
	    validate();
	}

	/** Gives notification that there was an insert into the document. */
	public void insertUpdate(DocumentEvent e) {
	    validate();
	}

	/** Gives notification that a portion of the document has been removed. */
	public void removeUpdate(DocumentEvent e) {
	    validate();
	}


	/** Ensures that the value text field and value tree are synchronized. */
	private void validate()
	{
	    String value_text = value.getText();
	    // System.err.println("\n(Validate) Value text: " + value_text);

	    // Ignore the validate() with empty text that occurs when value.setText() is used 
	    if (!value_text.equals("") || manual_text_edit_event == true) {
		TreePath closest_path = getClosestPath(value_text);

		// Select the new path in the tree
		// The tree selection event this causes must be ignored, since it alters value
		ignore_tree_selection_event = true;
		tree.setSelectionPath(closest_path);
		ignore_tree_selection_event = false;

		// If a folder has been specified, make sure it is expanded
		if (value_text.endsWith("\\") && !tree.isExpanded(closest_path)) {
		    tree.expandPath(closest_path);
		}

		// Make sure the path is visible on the screen
		scrollTreePathToVisible(closest_path);
		// One call should be enough, but it isn't in some cases (for some reason)
		scrollTreePathToVisible(closest_path);

		// Update the status of the buttons
		metaedit.validateControls();
	    }
	}
    }
}
